from __future__ import print_function

import os

import pool
import rcExceptions as ex
from rcUtilities import lazy, justcall
from rcGlobalEnv import rcEnv

class Pool(pool.Pool):
    type = "share"
    capabilities = ["rox", "rwx", "roo", "rwo", "blk", "shared"]

    @lazy
    def path(self):
        try:
            return self.node.conf_get(self.section, "path")
        except ex.OptNotFound as exc:
            return exc.default

    def translate_blk(self, path=None, size=None, shared=False):
        data = [
            {
                "rtype": "disk",
                "type": "loop",
                "file": "%s.img" % path,
                "size": size,
            }
        ]
        data += self.add_sync_internal(data)
        return data

    def translate(self, name=None, size=None, fmt=True, shared=False):
        if shared:
            path = os.path.join(self.path, name)
        else:
            path = os.path.join(self.path, "%s.{nodename}" % name)
        if not fmt:
            return self.translate_blk(path, size=size, shared=shared)

        fs = {
            "rtype": "fs",
            "type": "directory",
            "path": path,
        }
        return [fs]

    def status(self):
        from converters import convert_size
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        data = {
            "type": self.type,
            "name": self.name,
            "capabilities": self.capabilities,
            "head": self.path,
        }
        cmd = ["df", "-P", self.path]
        out, err, ret = justcall(cmd)
        if ret != 0:
            return data
        l = out.splitlines()[-1].split()
        data["free"] = int(l[3])
        data["used"] = int(l[2])
        data["size"] = int(l[1])
        return data

